@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.c 2974 2008-05-28 18:32:52Z rousseau $
+ * $Id: ccid.c 3280 2009-01-13 19:30:21Z rousseau $
  */
 
 #include <stdio.h>
@@ -222,6 +222,7 @@ int ccid_open_hack(unsigned int reader_index)
 						cmd[offset++] = ' ';
 				}
 
+				(void)sleep(1);
 				if (IFD_SUCCESS == CmdEscape(reader_index, cmd, sizeof(cmd), res, &length_res))
 				{
 					DEBUG_COMM("l10n string loaded successfully");
@@ -233,6 +234,7 @@ int ccid_open_hack(unsigned int reader_index)
 			}
 			break;
 
+#if 0
 		/* SCM SCR331-DI contactless */
 		case SCR331DI:
 		/* SCM SCR331-DI-NTTCOM contactless */
@@ -269,6 +271,7 @@ int ccid_open_hack(unsigned int reader_index)
 				ccid_descriptor->dwFeatures |= CCID_CLASS_AUTO_IFSD;
 			}
 			break;
+#endif
 
 		case MYSMARTPAD:
 			ccid_descriptor->dwMaxIFSD = 254;
@@ -276,13 +279,13 @@ int ccid_open_hack(unsigned int reader_index)
 
 		case CL1356D:
 			/* the firmware needs some time to initialize */
-			sleep(1);
+			(void)sleep(1);
 			ccid_descriptor->readTimeout = 60; /* 60 seconds */
 			break;
 
 		case SEG:
 #ifndef TWIN_SERIAL
-			InterruptRead(reader_index);
+			(void)InterruptRead(reader_index);
 #endif
 			break;
 	}
@@ -294,9 +297,9 @@ int ccid_open_hack(unsigned int reader_index)
 		unsigned int n = sizeof(tmp);
 
 		DEBUG_COMM("ICCD type A");
-		CmdPowerOff(reader_index);
-		CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
-		CmdPowerOff(reader_index);
+		(void)CmdPowerOff(reader_index);
+		(void)CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
+		(void)CmdPowerOff(reader_index);
 	}
 
 	/* ICCD type B */
@@ -314,9 +317,9 @@ int ccid_open_hack(unsigned int reader_index)
 			ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
 		}
 
-		CmdPowerOff(reader_index);
-		CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
-		CmdPowerOff(reader_index);
+		(void)CmdPowerOff(reader_index);
+		(void)CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
+		(void)CmdPowerOff(reader_index);
 	}
 
 	return 0;
@@ -437,11 +440,13 @@ void ccid_error(int error, const char *file, int line, const char *function)
 		default:
 			if ((error >= 1) && (error <= 127))
 			{
-				sprintf(var_text, "error on byte %d", error);
+				(void)snprintf(var_text, sizeof(var_text), "error on byte %d",
+					error);
 				text = var_text;
 			}
 			else
-				sprintf(var_text, "Unknown CCID error: 0x%02X", error);
+				(void)snprintf(var_text, sizeof(var_text),
+					"Unknown CCID error: 0x%02X", error);
 				text = var_text;
 			break;
 	}
