@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid.h 2974 2008-05-28 18:32:52Z rousseau $
+ * $Id: ccid.h 3207 2008-11-18 15:02:50Z rousseau $
  */
 
 typedef struct
@@ -197,7 +197,7 @@ void ccid_error(int error, const char *file, int line, const char *function);
 _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index);
 
 /* convert a 4 byte integer in USB format into an int */
-#define dw2i(a, x) ((((((a[x+3] << 8) + a[x+2]) << 8) + a[x+1]) << 8) + a[x])
+#define dw2i(a, x) (unsigned int)((((((a[x+3] << 8) + a[x+2]) << 8) + a[x+1]) << 8) + a[x])
 
 /* all the data rates specified by ISO 7816-3 Fi/Di tables */
 #define ISO_DATA_RATES 10753, 14337, 15625, 17204, \
