@@ -20,7 +20,7 @@
  */
 
 /*
- * $Id: ccid_serial.c 2983 2008-06-08 08:31:47Z rousseau $
+ * $Id: ccid_serial.c 3271 2009-01-05 14:56:38Z rousseau $
  */
 
 #include <stdio.h>
@@ -160,7 +160,7 @@ static _serialDevice serialDevice[CCID_DRIVER_MAX_READERS];
 static int ReadChunk(unsigned int reader_index, unsigned char *buffer,
 	int buffer_length, int min_length);
 
-static int get_bytes(unsigned int reader_index, unsigned char *buffer,
+static int get_bytes(unsigned int reader_index, /*@out@*/ unsigned char *buffer,
 	int length);
 
 
@@ -178,7 +178,8 @@ status_t WriteSerial(unsigned int reader_index, unsigned int length,
 
 	char debug_header[] = "-> 123456 ";
 
-	sprintf(debug_header, "-> %06X ", reader_index);
+	(void)snprintf(debug_header, sizeof(debug_header), "-> %06X ",
+		reader_index);
 
 	if (length > GEMPCTWIN_MAXBUF-3)
 	{
@@ -411,7 +412,8 @@ static int ReadChunk(unsigned int reader_index, unsigned char *buffer,
 	int already_read;
 	char debug_header[] = "<- 123456 ";
 
-	sprintf(debug_header, "<- %06X ", reader_index);
+	(void)snprintf(debug_header, sizeof(debug_header), "<- %06X ",
+		reader_index);
 
 	already_read = 0;
 	while (already_read < min_length)
@@ -485,7 +487,7 @@ status_t OpenSerial(unsigned int reader_index, int channel)
 		return STATUS_UNSUCCESSFUL;
 	}
 
-	sprintf(dev_name, "/dev/pcsc/%d", (int) channel);
+	(void)snprintf(dev_name, sizeof(dev_name), "/dev/pcsc/%d", (int) channel);
 
 	return OpenSerialByName(reader_index, dev_name);
 } /* OpenSerial */
@@ -503,7 +505,7 @@ status_t OpenSerial(unsigned int reader_index, int channel)
 static status_t set_ccid_descriptor(unsigned int reader_index,
 	const char *reader_name, const char *dev_name)
 {
-	long readerID;
+	int readerID;
 	int i;
 	int already_used = FALSE;
 	static int previous_reader_index = -1;
@@ -705,7 +707,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 	if (tcgetattr(serialDevice[reader].fd, &current_termios) == -1)
 	{
 		DEBUG_INFO2("tcgetattr() function error: %s", strerror(errno));
-		close(serialDevice[reader].fd);
+		(void)close(serialDevice[reader].fd);
 		serialDevice[reader].fd = -1;
 
 		return STATUS_UNSUCCESSFUL;
@@ -726,12 +728,12 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 	current_termios.c_lflag = 0;
 
 	/* set serial port speed to 115200 bauds */
-	cfsetspeed(&current_termios, B115200);
+	(void)cfsetspeed(&current_termios, B115200);
 
 	DEBUG_INFO("Set serial port baudrate to 115200 and correct configuration");
 	if (tcsetattr(serialDevice[reader].fd, TCSANOW, &current_termios) == -1)
 	{
-		close(serialDevice[reader].fd);
+		(void)close(serialDevice[reader].fd);
 		serialDevice[reader].fd = -1;
 		DEBUG_INFO2("tcsetattr error: %s", strerror(errno));
 
@@ -752,7 +754,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 			rx_buffer, &rx_length))
 		{
 			DEBUG_CRITICAL("Get firmware failed. Maybe the reader is not connected");
-			CloseSerial(reader_index);
+			(void)CloseSerial(reader_index);
 			return STATUS_UNSUCCESSFUL;
 		}
 
@@ -775,7 +777,7 @@ status_t OpenSerialByName(unsigned int reader_index, char *dev_name)
 			rx_buffer, &rx_length))
 		{
 			DEBUG_CRITICAL("Change card movement notification failed.");
-			CloseSerial(reader_index);
+			(void)CloseSerial(reader_index);
 			return STATUS_UNSUCCESSFUL;
 		}
 	}
@@ -807,7 +809,7 @@ status_t CloseSerial(unsigned int reader_index)
 	{
 		DEBUG_COMM("Last slot closed. Release resources");
 
-		close(serialDevice[reader].fd);
+		(void)close(serialDevice[reader].fd);
 		serialDevice[reader].fd = -1;
 
 		free(serialDevice[reader].device);
