@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: defs.h 3006 2008-06-17 12:35:15Z rousseau $
+ * $Id: defs.h 3199 2008-11-10 12:12:25Z rousseau $
  */
 
 #include <pcsclite.h>
@@ -42,6 +42,9 @@ typedef struct CCID_DESC
 	 * T=1 Protocol context
 	 */
 	t1_state_t t1;
+
+	/* reader name passed to IFDHCreateChannelByName() */
+	char *readerName;
 } CcidDesc;
 
 typedef enum {
