@@ -17,7 +17,7 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/* $Id: ifdhandler.c 3025 2008-06-26 13:38:50Z rousseau $ */
+/* $Id: ifdhandler.c 3337 2009-03-07 14:30:17Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -90,8 +90,11 @@ EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
 	/* Reset PowerFlags */
 	CcidSlots[reader_index].bPowerFlags = POWERFLAGS_RAZ;
 
+	/* reader name */
+	CcidSlots[reader_index].readerName = strdup(lpcDevice);
+
 #ifdef HAVE_PTHREAD
-	pthread_mutex_lock(&ifdh_context_mutex);
+	(void)pthread_mutex_lock(&ifdh_context_mutex);
 #endif
 
 	if (OpenPortByName(reader_index, lpcDevice) != STATUS_SUCCESS)
@@ -105,7 +108,7 @@ EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
 	else
 	{
 		/* Maybe we have a special treatment for this reader */
-		ccid_open_hack(reader_index);
+		(void)ccid_open_hack(reader_index);
 
 		/* Try to access the reader */
 		/* This "warm up" sequence is sometimes needed when pcscd is
@@ -119,13 +122,14 @@ EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
 			DEBUG_CRITICAL("failed");
 			return_value = IFD_COMMUNICATION_ERROR;
 
-			/* release the allocated reader_index */
+			/* release the allocated resources */
+			(void)ClosePort(reader_index);
 			ReleaseReaderIndex(reader_index);
 		}
 	}
 
 #ifdef HAVE_PTHREAD
-	pthread_mutex_unlock(&ifdh_context_mutex);
+	(void)pthread_mutex_unlock(&ifdh_context_mutex);
 #endif
 
 	return return_value;
@@ -185,8 +189,11 @@ EXTERNAL RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
 	/* Reset PowerFlags */
 	CcidSlots[reader_index].bPowerFlags = POWERFLAGS_RAZ;
 
+	/* reader name */
+	CcidSlots[reader_index].readerName = strdup("no name");
+
 #ifdef HAVE_PTHREAD
-	pthread_mutex_lock(&ifdh_context_mutex);
+	(void)pthread_mutex_lock(&ifdh_context_mutex);
 #endif
 
 	if (OpenPort(reader_index, Channel) != STATUS_SUCCESS)
@@ -199,10 +206,10 @@ EXTERNAL RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
 	}
 	else
 		/* Maybe we have a special treatment for this reader */
-		ccid_open_hack(reader_index);
+		(void)ccid_open_hack(reader_index);
 
 #ifdef HAVE_PTHREAD
-	pthread_mutex_unlock(&ifdh_context_mutex);
+	(void)pthread_mutex_unlock(&ifdh_context_mutex);
 #endif
 
 	return return_value;
@@ -223,11 +230,11 @@ EXTERNAL RESPONSECODE IFDHCloseChannel(DWORD Lun)
 	 */
 	int reader_index;
 
-	DEBUG_INFO2("lun: %X", Lun);
-
 	if (-1 == (reader_index = LunToReaderIndex(Lun)))
 		return IFD_COMMUNICATION_ERROR;
 
+	DEBUG_INFO3("%s (lun: %X)", CcidSlots[reader_index].readerName, Lun);
+
 	/* Restore the default timeout
 	 * No need to wait too long if the reader disapeared */
 	get_ccid_descriptor(reader_index)->readTimeout = DEFAULT_COM_READ_TIMEOUT;
@@ -236,14 +243,17 @@ EXTERNAL RESPONSECODE IFDHCloseChannel(DWORD Lun)
 	/* No reader status check, if it failed, what can you do ? :) */
 
 #ifdef HAVE_PTHREAD
-	pthread_mutex_lock(&ifdh_context_mutex);
+	(void)pthread_mutex_lock(&ifdh_context_mutex);
 #endif
 
 	(void)ClosePort(reader_index);
 	ReleaseReaderIndex(reader_index);
 
+	free(CcidSlots[reader_index].readerName);
+	memset(&CcidSlots[reader_index], 0, sizeof(CcidSlots[reader_index]));
+
 #ifdef HAVE_PTHREAD
-	pthread_mutex_unlock(&ifdh_context_mutex);
+	(void)pthread_mutex_unlock(&ifdh_context_mutex);
 #endif
 
 	return IFD_SUCCESS;
@@ -261,7 +271,7 @@ static RESPONSECODE IFDHPolling(DWORD Lun)
 
 	/* log only if DEBUG_LEVEL_PERIODIC is set */
 	if (LogLevel & DEBUG_LEVEL_PERIODIC)
-		DEBUG_INFO2("lun: %X", Lun);
+		DEBUG_INFO3("%s (lun: %X)", CcidSlots[reader_index].readerName, Lun);
 
 	ret = InterruptRead(reader_index);
 	if (ret > 0)
@@ -275,8 +285,21 @@ static RESPONSECODE IFDHPolling(DWORD Lun)
  * so no card movement will ever happen: just do nothing */
 static RESPONSECODE IFDHSleep(DWORD Lun)
 {
-	DEBUG_INFO2("lun: %X", Lun);
-	sleep(60*60);	/* 1 hour */
+	int reader_index;
+
+	if (-1 == (reader_index = LunToReaderIndex(Lun)))
+		return IFD_COMMUNICATION_ERROR;
+
+	DEBUG_INFO3("%s (lun: %X)", CcidSlots[reader_index].readerName, Lun);
+
+	/* just sleep for 5 seconds since the polling thread is NOT killable
+	 * so pcscd event thread must loop to exit cleanly
+	 *
+	 * Once the driver (libusb in fact) will support
+	 * TAG_IFD_POLLING_THREAD_KILLABLE then we could use a much longer delay
+	 * and be killed before pcscd exits
+	 */
+	(void)sleep(600);	/* 10 minutes */
 	return IFD_SUCCESS;
 }
 #endif
@@ -304,11 +327,12 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 	 */
 	int reader_index;
 
-	DEBUG_INFO3("lun: %X, tag: 0x%X", Lun, Tag);
-
 	if (-1 == (reader_index = LunToReaderIndex(Lun)))
 		return IFD_COMMUNICATION_ERROR;
 
+	DEBUG_INFO4("tag: 0x%X, %s (lun: %X)", Tag,
+		CcidSlots[reader_index].readerName, Lun);
+
 	switch (Tag)
 	{
 		case TAG_IFD_ATR:
@@ -412,6 +436,24 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 				}
 			}
 			break;
+
+		case TAG_IFD_POLLING_THREAD_KILLABLE:
+			{
+				_ccid_descriptor *ccid_desc;
+
+				/* default value: not supported */
+				*Length = 0;
+
+				ccid_desc = get_ccid_descriptor(reader_index);
+				if ((ICCD_A == ccid_desc->bInterfaceProtocol)
+					|| (ICCD_B == ccid_desc->bInterfaceProtocol))
+				{
+					*Length = 1;	/* 1 char */
+					if (Value)
+						*Value = 1;	/* TRUE */
+				}
+			}
+			break;
 #endif
 
 		default:
@@ -444,7 +486,13 @@ EXTERNAL RESPONSECODE IFDHSetCapabilities(DWORD Lun, DWORD Tag,
 
 	/* By default, say it worked */
 
-	DEBUG_INFO3("lun: %X, tag: 0x%X", Lun, Tag);
+	int reader_index;
+
+	if (-1 == (reader_index = LunToReaderIndex(Lun)))
+		return IFD_COMMUNICATION_ERROR;
+
+	DEBUG_INFO4("tag: 0x%X, %s (lun: %X)", Tag,
+		CcidSlots[reader_index].readerName, Lun);
 
 	/* if (CheckLun(Lun))
 		return IFD_COMMUNICATION_ERROR; */
@@ -460,7 +508,7 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	 * This function should set the PTS of a particular card/slot using
 	 * the three PTS parameters sent
 	 *
-	 * Protocol - 0 .... 14 T=0 .... T=14
+	 * Protocol - SCARD_PROTOCOL_T0 or SCARD_PROTOCOL_T1
 	 * Flags - Logical OR of possible values:
 	 *  IFD_NEGOTIATE_PTS1
 	 *  IFD_NEGOTIATE_PTS2
@@ -485,11 +533,12 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	CcidDesc *ccid_slot;
 	_ccid_descriptor *ccid_desc;
 
-	DEBUG_INFO3("lun: %X, protocol T=%d", Lun, Protocol-1);
-
 	if (-1 == (reader_index = LunToReaderIndex(Lun)))
 		return IFD_COMMUNICATION_ERROR;
 
+	DEBUG_INFO4("protocol T=%d, %s (lun: %X)", Protocol-SCARD_PROTOCOL_T0,
+		CcidSlots[reader_index].readerName, Lun);
+
 	/* Set to zero buffer */
 	memset(pps, 0, sizeof(pps));
 	memset(&atr, 0, sizeof(atr));
@@ -504,7 +553,8 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		goto end;
 
 	/* Get ATR of the card */
-	ATR_InitFromArray(&atr, ccid_slot->pcATRBuffer, ccid_slot->nATRLength);
+	(void)ATR_InitFromArray(&atr, ccid_slot->pcATRBuffer,
+		ccid_slot->nATRLength);
 
 	/* Apply Extra EGT patch for bogus cards */
 	extra_egt(&atr, ccid_desc, Protocol);
@@ -543,13 +593,13 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 				if (0 == atr.ib[i][ATR_INTERFACE_BYTE_TC].value)
 				{
 					DEBUG_COMM("Use LRC");
-					t1_set_param(t1, IFD_PROTOCOL_T1_CHECKSUM_LRC, 0);
+					(void)t1_set_param(t1, IFD_PROTOCOL_T1_CHECKSUM_LRC, 0);
 				}
 				else
 					if (1 == atr.ib[i][ATR_INTERFACE_BYTE_TC].value)
 					{
 						DEBUG_COMM("Use CRC");
-						t1_set_param(t1, IFD_PROTOCOL_T1_CHECKSUM_CRC, 0);
+						(void)t1_set_param(t1, IFD_PROTOCOL_T1_CHECKSUM_CRC, 0);
 					}
 					else
 						DEBUG_COMM2("Wrong value for TCi: %d",
@@ -576,8 +626,8 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			unsigned int default_baudrate;
 			double f, d;
 
-			ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
-			ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
+			(void)ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
+			(void)ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
 
 			/* may happen with non ISO cards */
 			if ((0 == f) || (0 == d))
@@ -639,8 +689,8 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 						/* use a lower TA1 */
 						atr.ib[0][ATR_INTERFACE_BYTE_TA].value--;
 
-						ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
-						ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
+						(void)ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
+						(void)ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
 
 						/* Baudrate = f x D/F */
 						card_baudrate = (unsigned int) (1000 *
@@ -701,9 +751,11 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			if ((OZ776 == ccid_desc->readerID)
 				|| (OZ776_7772 == ccid_desc->readerID))
 			{
-				Protocol = default_protocol;
+				/* convert from ATR_PROTOCOL_TYPE_T? to SCARD_PROTOCOL_T? */
+				Protocol = default_protocol +
+				   	(SCARD_PROTOCOL_T0 - ATR_PROTOCOL_TYPE_T0);
 				DEBUG_INFO2("PPS not supported on O2Micro readers. Using T=%d",
-					Protocol);
+					Protocol - SCARD_PROTOCOL_T0);
 			}
 			else
 #endif
@@ -717,7 +769,7 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	}
 
 	/* Now we must set the reader parameters */
-	ATR_GetConvention(&atr, &convention);
+	(void)ATR_GetConvention(&atr, &convention);
 
 	/* specific mode and implicit parameters? (b5 of TA2) */
 	if (atr.ib[1][ATR_INTERFACE_BYTE_TA].present
@@ -765,13 +817,30 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 					atr.ib[i][ATR_INTERFACE_BYTE_TB].value);
 				param[3] = atr.ib[i][ATR_INTERFACE_BYTE_TB].value;
 
+				{
+					/* Hack for OpenPGP card */
+					char openpgp_atr[] = { 0x3B, 0xFA, 0x13, 0x00, 0xFF, 0x81,
+						0x31, 0x80, 0x45, 0x00, 0x31, 0xC1, 0x73, 0xC0,
+						0x01, 0x00, 0x00, 0x90, 0x00, 0xB1 };
+
+					if (0 == memcmp(ccid_slot->pcATRBuffer, openpgp_atr,
+						ccid_slot->nATRLength))
+						/* change BWI from 4 to 7 to increase BWT from
+						 * 1.4s to 11s and avoid a timeout during on
+						 * board key generation (bogus card) */
+					{
+						param[3] = 0x75;
+						DEBUG_COMM2("OpenPGP hack, using 0x%02X", param[3]);
+					}
+				}
+
 				/* only the first TBi (i>2) must be used */
 				break;
 			}
 
 		/* compute communication timeout */
-		ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
-		ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
+		(void)ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
+		(void)ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
 		ccid_desc->readTimeout = T1_card_timeout(f, d, param[2],
 			(param[3] & 0xF0) >> 4 /* BWI */, param[3] & 0x0F /* CWI */,
 			ccid_desc->dwDefaultClock);
@@ -823,8 +892,8 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			param[3] = atr.ib[1][ATR_INTERFACE_BYTE_TC].value;
 
 		/* compute communication timeout */
-		ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
-		ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
+		(void)ATR_GetParameter(&atr, ATR_PARAMETER_F, &f);
+		(void)ATR_GetParameter(&atr, ATR_PARAMETER_D, &d);
 
 		ccid_desc->readTimeout = T0_card_timeout(f, d, param[2] /* TC1 */,
 			param[3] /* TC2 */, ccid_desc->dwDefaultClock);
@@ -849,7 +918,7 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			{
 				DEBUG_COMM3("IFSC (TA%d) present: %d", i+1,
 					atr.ib[i][ATR_INTERFACE_BYTE_TA].value);
-				t1_set_param(t1, IFD_PROTOCOL_T1_IFSC,
+				(void)t1_set_param(t1, IFD_PROTOCOL_T1_IFSC,
 					atr.ib[i][ATR_INTERFACE_BYTE_TA].value);
 
 				/* only the first TAi (i>2) must be used */
@@ -860,10 +929,10 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		if (! (ccid_desc->dwFeatures & CCID_CLASS_AUTO_IFSD))
 		{
 			DEBUG_COMM2("Negociate IFSD at %d", ccid_desc -> dwMaxIFSD);
-			if (t1_negociate_ifsd(t1, 0, ccid_desc -> dwMaxIFSD) < 0)
+			if (t1_negotiate_ifsd(t1, 0, ccid_desc -> dwMaxIFSD) < 0)
 				return IFD_COMMUNICATION_ERROR;
 		}
-		t1_set_param(t1, IFD_PROTOCOL_T1_IFSD, ccid_desc -> dwMaxIFSD);
+		(void)t1_set_param(t1, IFD_PROTOCOL_T1_IFSD, ccid_desc -> dwMaxIFSD);
 
 		DEBUG_COMM3("T=1: IFSC=%d, IFSD=%d", t1->ifsc, t1->ifsd);
 	}
@@ -920,8 +989,8 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 	unsigned char pcbuffer[RESP_BUF_SIZE];
 	int reader_index;
 	const char *actions[] = { "PowerUp", "PowerDown", "Reset" };
-
-	DEBUG_INFO3("lun: %X, action: %s", Lun, actions[Action-IFD_POWER_UP]);
+	unsigned int oldReadTimeout;
+	_ccid_descriptor *ccid_descriptor;
 
 	/* By default, assume it won't work :) */
 	*AtrLength = 0;
@@ -929,6 +998,9 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 	if (-1 == (reader_index = LunToReaderIndex(Lun)))
 		return IFD_COMMUNICATION_ERROR;
 
+	DEBUG_INFO4("action: %s, %s (lun: %X)", actions[Action-IFD_POWER_UP],
+		CcidSlots[reader_index].readerName, Lun);
+
 	switch (Action)
 	{
 		case IFD_POWER_DOWN:
@@ -953,9 +1025,28 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 
 		case IFD_POWER_UP:
 		case IFD_RESET:
+			/* save the current read timeout computed from card capabilities */
+			ccid_descriptor = get_ccid_descriptor(reader_index);
+			oldReadTimeout = ccid_descriptor->readTimeout;
+
+			/* use a very long timeout since the card can use up to
+			 * (9600+12)*33 ETU in total
+			 * 12 ETU per byte
+			 * 9600 ETU max between each byte
+			 * 33 bytes max for ATR
+			 * 1 ETU = 372 cycles during ATR
+			 * with a 4 MHz clock => 29 seconds
+			 */
+			ccid_descriptor->readTimeout = 60;
+
 			nlength = sizeof(pcbuffer);
-			if (CmdPowerOn(reader_index, &nlength, pcbuffer, PowerOnVoltage)
-				!= IFD_SUCCESS)
+			return_value = CmdPowerOn(reader_index, &nlength, pcbuffer,
+				PowerOnVoltage);
+
+			/* set back the old timeout */
+			ccid_descriptor->readTimeout = oldReadTimeout;
+
+			if (return_value != IFD_SUCCESS)
 			{
 				/* used by GemCore SIM PRO: no card is present */
 				get_ccid_descriptor(reader_index)->dwSlotStatus
@@ -977,7 +1068,7 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 			memcpy(CcidSlots[reader_index].pcATRBuffer, pcbuffer, *AtrLength);
 
 			/* initialise T=1 context */
-			t1_init(&(get_ccid_slot(reader_index) -> t1), reader_index);
+			(void)t1_init(&(get_ccid_slot(reader_index) -> t1), reader_index);
 			break;
 
 		default:
@@ -1032,11 +1123,11 @@ EXTERNAL RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 	unsigned int rx_length;
 	int reader_index;
 
-	DEBUG_INFO2("lun: %X", Lun);
-
 	if (-1 == (reader_index = LunToReaderIndex(Lun)))
 		return IFD_COMMUNICATION_ERROR;
 
+	DEBUG_INFO3("%s (lun: %X)", CcidSlots[reader_index].readerName, Lun);
+
 	rx_length = *RxLength;
 	return_value = CmdXfrBlock(reader_index, TxLength, TxBuffer, &rx_length,
 		RxBuffer, SendPci.Protocol);
@@ -1070,13 +1161,14 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 	RESPONSECODE return_value = IFD_COMMUNICATION_ERROR;
 	int reader_index;
 
-	DEBUG_INFO3("lun: %X, ControlCode: 0x%X", Lun, dwControlCode);
-	DEBUG_INFO_XXD("Control TxBuffer: ", TxBuffer, TxLength);
-
 	reader_index = LunToReaderIndex(Lun);
 	if ((-1 == reader_index) || (NULL == pdwBytesReturned))
 		return return_value;
 
+	DEBUG_INFO4("ControlCode: 0x%X, %s (lun: %X)", dwControlCode,
+		CcidSlots[reader_index].readerName, Lun);
+	DEBUG_INFO_XXD("Control TxBuffer: ", TxBuffer, TxLength);
+
 	/* Set the return length to 0 to avoid problems */
 	*pdwBytesReturned = 0;
 
@@ -1183,11 +1275,11 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 	_ccid_descriptor *ccid_descriptor;
 	unsigned int oldReadTimeout;
 
-	DEBUG_PERIODIC2("lun: %X", Lun);
-
 	if (-1 == (reader_index = LunToReaderIndex(Lun)))
 		return IFD_COMMUNICATION_ERROR;
 
+	DEBUG_PERIODIC3("%s (lun: %X)", CcidSlots[reader_index].readerName, Lun);
+
 	ccid_descriptor = get_ccid_descriptor(reader_index);
 
 	if (GEMCORESIMPRO == ccid_descriptor->readerID)
@@ -1253,6 +1345,7 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 			break;
 	}
 
+#if 0
 	/* SCR331-DI contactless reader */
 	if (((SCR331DI == ccid_descriptor->readerID)
 		|| (SDI010 == ccid_descriptor->readerID)
@@ -1299,6 +1392,7 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 			return_value = IFD_ICC_NOT_PRESENT;
 		}
 	}
+#endif
 
 end:
 	DEBUG_PERIODIC2("Card %s",
@@ -1323,7 +1417,7 @@ void init_driver(void)
 	DEBUG_INFO("Driver version: " VERSION);
 
 	/* Info.plist full patch filename */
-	snprintf(infofile, sizeof(infofile), "%s/%s/Contents/Info.plist",
+	(void)snprintf(infofile, sizeof(infofile), "%s/%s/Contents/Info.plist",
 		PCSCLITE_HP_DROPDIR, BUNDLE);
 
 	/* Log level */
@@ -1406,8 +1500,8 @@ void extra_egt(ATR_t *atr, _ccid_descriptor *ccid_desc, DWORD Protocol)
 	if (! atr->ib[0][ATR_INTERFACE_BYTE_TA].present)
 		return;
 
-	ATR_GetParameter(atr, ATR_PARAMETER_D, &d);
-	ATR_GetParameter(atr, ATR_PARAMETER_F, &f);
+	(void)ATR_GetParameter(atr, ATR_PARAMETER_D, &d);
+	(void)ATR_GetParameter(atr, ATR_PARAMETER_F, &f);
 
 	/* may happen with non ISO cards */
 	if ((0 == f) || (0 == d))
@@ -1528,7 +1622,7 @@ static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2,
 	/* ISO in */
 	t  = 261 * EGT + (3 + 3) * WWT;
 	/* Convert from milliseonds to seconds rouned to the upper value
-	 * use +1 instead of ceil() to round up to the nearest interger
+	 * use +1 instead of ceil() to round up to the nearest integer
 	 * so we can avoid a dependency on the math library */
 	t = t/1000 +1;
 	if (timeout < t)
@@ -1585,7 +1679,7 @@ static unsigned int T1_card_timeout(double f, double d, int TC1,
 	timeout = 260*EGT + BWT + 260*CWT;
 
 	/* Convert from milliseonds to seconds rounded to the upper value
-	 * we use +1 instead of ceil() to round up to the nearest greater interger
+	 * we use +1 instead of ceil() to round up to the nearest greater integer
 	 * so we can avoid a dependency on the math library */
 	timeout = timeout/1000 +1;
 
