@@ -12,13 +12,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: utils.h 2121 2006-08-03 13:12:20Z rousseau $
+ * $Id: utils.h 2974 2008-05-28 18:32:52Z rousseau $
  */
 
 #ifndef TRUE
@@ -27,7 +27,7 @@
 #endif
 
 void InitReaderIndex(void);
-int GetNewReaderIndex(const DWORD Lun);
-int LunToReaderIndex(DWORD Lun);
+int GetNewReaderIndex(const int Lun);
+int LunToReaderIndex(int Lun);
 void ReleaseReaderIndex(const int index);
 
